
#!/usr/bin/perl
# NOTE - THIS FILE GENERATED BY DBIxGenerate.pl.
# To regenerate this file, simply call the script, and it'll dump the DBIx files into the working directory.

=head1 NAME

WWW::Shopify::Model::DBIx::Schema::Result::Model::DBIx::Schema::Result::Model::MetafieldProduct - Database Object for WWW::Shopify::Model::DBIx::Schema::Result::Model::MetafieldProduct.

=cut

=head1 NOTICE

This is a generated class, made by WWW::Shopify/Utils/DBIxGenerate.pl. Do not modify.

=cut

use strict;
use warnings;


			package WWW::Shopify::Model::DBIx::Schema::Result::Model::MetafieldProduct;
			use base qw/DBIx::Class::Core/;

			__PACKAGE__->table('shopify_metafieldproduct');
			__PACKAGE__->add_columns(
				'product_id', { data_type => 'int', is_nullable => 0 },
				'metafield_id', { data_type => 'int', is_nullable => 0 });
			__PACKAGE__->belongs_to(product => 'WWW::Shopify::Model::DBIx::Schema::Result::Model::Product', 'product_id');
			__PACKAGE__->belongs_to(metafield => 'WWW::Shopify::Model::DBIx::Schema::Result::Model::Metafield', 'metafield_id');

1;